@@ -117,9 +117,8 @@ typedef struct timer_info {
 	struct timer_info *hash_next;	/* Next in hash list */
 	pid_t		pid;
 	uint32_t	ref_count;	/* Timer stat reference count */
-	char 		*task;		/* Name of process/kernel task */
-	char 		*task_mangled;	/* Modified name of process/kernel */
 	char		*cmdline;	/* From /proc/$pid/cmdline */
+	char		*comm;		/* from /proc/$pid/comm */
 	char		*func;		/* Kernel waiting func */
 	char		*ident;		/* Unique identity */
 	uint64_t	timer;		/* Timer ID */
@@ -710,7 +709,8 @@ static void sample_add(timer_stat_t *timer_stat, const double whence)
 	 * list since time is assumed to be increasing
 	 */
 	if (!found) {
-		if ((sdl = calloc(1, sizeof(*sdl))) == NULL)
+		sdl = calloc(1, sizeof(*sdl));
+		if (UNLIKELY(!sdl))
 			err_abort("Cannot allocate sample delta list\n");
 		sdl->whence = whence;
 
@@ -724,7 +724,8 @@ static void sample_add(timer_stat_t *timer_stat, const double whence)
 	}
 
 	/* Now append the sdi onto the list */
-	if (UNLIKELY(((sdi = calloc(1, sizeof(*sdi))) == NULL)))
+	sdi = calloc(1, sizeof(*sdi));
+	if (UNLIKELY(!sdi))
 		err_abort("Cannot allocate sample delta item\n");
 	sdi->delta_events = timer_stat->info->delta_events;
 	sdi->time_delta = timer_stat->info->last_used -
@@ -826,9 +827,60 @@ static bool pid_a_kernel_thread(const char *task, const pid_t id)
 	return pid_a_kernel_thread_guess(task);
 }
 
+/*
+ *  unknown_comm()
+ *	return an unknonw comm field replacement
+ */
+static char *unknown_comm(void)
+{
+	return strdup("<unknown>");
+}
+
+/*
+ *  get_pid_comm
+ *	get process /proc/pid/comm field info
+ */
+static char *get_pid_comm(const pid_t id, bool kernel_thread)
+{
+	char buffer[256];
+	ssize_t ret;
+	char *comm, *ptr;
+	int fd;
+
+	(void)snprintf(buffer, sizeof(buffer), "/proc/%d/comm", id);
+	if ((fd = open(buffer, O_RDONLY)) < 0)
+		return unknown_comm();
+
+	ret = read(fd, buffer, sizeof(buffer));
+	(void)close(fd);
+	if (ret <= 0)
+		return unknown_comm();
+
+	buffer[sizeof(buffer)-1] = '\0';
+	for (ptr = buffer; *ptr; ptr++) {
+		if (*ptr == '\n') {
+			*ptr = '\0';
+			break;
+		}
+	}
+
+	if (kernel_thread) {
+		size_t len = strlen(buffer) + 3;
+		comm = malloc(len);
+		if (UNLIKELY(!comm))
+			return unknown_comm();
+		snprintf(comm, len, "[%s]", buffer);
+	} else {
+		comm = strdup(buffer);
+		if (!comm)
+			return unknown_comm();
+	}
+	return comm;
+}
+
 /*
  *  get_pid_cmdline
- *	get process's /proc/pid/cmdline
+ *	get process /proc/pid/cmdline
  */
 static char *get_pid_cmdline(const pid_t id)
 {
@@ -871,8 +923,13 @@ static char *get_pid_cmdline(const pid_t id)
 		}
 	}
 
-	if (g_opt_flags & OPT_DIRNAME_STRIP)
-		return strdup(basename(buffer));
+	if (g_opt_flags & OPT_DIRNAME_STRIP) {
+		char *base = basename(buffer);
+
+		/* Should always be true */
+		if (base)
+			return strdup(base);
+	}
 
 	return strdup(buffer);
 }
@@ -906,7 +963,7 @@ static void samples_dump(const char *filename)
 
 	sorted_timer_infos = calloc(g_timer_info_list_length,
 				sizeof(*sorted_timer_infos));
-	if (!sorted_timer_infos)
+	if (UNLIKELY(!sorted_timer_infos))
 		err_abort("Cannot allocate buffer for sorting timer_infos\n");
 
 	/* Just want the timers with some non-zero total */
@@ -925,7 +982,7 @@ static void samples_dump(const char *filename)
 		if (g_opt_flags & OPT_CMD)
 			task = sorted_timer_infos[i]->cmdline;
 		else
-			task = sorted_timer_infos[i]->task_mangled;
+			task = sorted_timer_infos[i]->comm;
 
 		(void)fprintf(fp, ",%s", task);
 	}
@@ -948,8 +1005,12 @@ static void samples_dump(const char *filename)
 
 		count++;
 		tm = localtime(&t);
-		(void)fprintf(fp, "%2.2d:%2.2d:%2.2d",
-			tm->tm_hour, tm->tm_min, tm->tm_sec);
+		if (tm) {
+			(void)fprintf(fp, "%2.2d:%2.2d:%2.2d",
+				tm->tm_hour, tm->tm_min, tm->tm_sec);
+		} else {
+			(void)fprintf(fp, "--:--:--");
+		}
 
 		if (first_time < 0)
 			first_time = sdl->whence;
@@ -1083,8 +1144,7 @@ static HOT timer_info_t *timer_info_find(
 		err_abort("Cannot allocate timer info\n");
 
 	info->pid = new_info->pid;
-	info->task = strdup(new_info->task);
-	info->task_mangled = strdup(new_info->task_mangled);
+	info->comm = new_info->comm ? strdup(new_info->comm) : unknown_comm();
 	info->cmdline = strdup(new_info->cmdline);
 	info->func = strdup(new_info->func);
 	info->ident = strdup(ident);
@@ -1099,8 +1159,7 @@ static HOT timer_info_t *timer_info_find(
 	get_proc_cpu_ticks(info->pid, &info->cpu_ticks, &info->cpu_rt_prio, &info->cpu_nice);
 	info->cpu_ticks_time = time_now;
 
-	if (UNLIKELY(info->task == NULL ||
-		     info->task_mangled == NULL ||
+	if (UNLIKELY(info->comm == NULL ||
 		     info->cmdline == NULL ||
 		     info->func == NULL ||
 		     info->ident == NULL)) {
@@ -1125,10 +1184,8 @@ static void timer_info_free(void *data)
 {
 	timer_info_t *info = (timer_info_t*)data;
 
-	free(info->task);
-	if (info->cmdline != info->task_mangled)
-		free(info->cmdline);
-	free(info->task_mangled);
+	free(info->comm);
+	free(info->cmdline);
 	free(info->func);
 	free(info->ident);
 	free(info);
@@ -1243,10 +1300,10 @@ static char *make_hash_ident(const timer_info_t *info)
 
 	if (g_opt_flags & OPT_TIMER_ID) {
 		(void)snprintf(ident, sizeof(ident), "%x%s%8.8s%" PRIx64,
-			info->pid, info->task, info->func, info->timer);
+			info->pid, info->comm, info->func, info->timer);
 	} else {
 		(void)snprintf(ident, sizeof(ident), "%x%s%8.8s",
-			info->pid, info->task, info->func);
+			info->pid, info->comm, info->func);
 	}
 	return ident;
 }
@@ -1324,7 +1381,8 @@ static void timer_stat_add(
 		g_timer_stat_free_list = g_timer_stat_free_list->next;
 	} else {
 		/* Get one from heap */
-		if ((ts_new = malloc(sizeof(*ts_new))) == NULL)
+		ts_new = malloc(sizeof(*ts_new));
+		if (UNLIKELY(!ts_new))
 			err_abort("Out of memory allocating a timer stat\n");
 	}
 
@@ -1483,10 +1541,7 @@ static OPTIMIZE3 void timer_stat_dump(
 			    (sorted->info->delta_events != 0)) {
 				char *task = (g_opt_flags & OPT_CMD) ?
 					sorted->info->cmdline :
-					sorted->info->task_mangled;
-				if (!*task)
-					task = sorted->info->task_mangled;
-
+					sorted->info->comm;
 				j++;
 				if (g_opt_flags & OPT_CUMULATIVE)
 					es_printf("%*" PRIu64 " ",
@@ -1552,9 +1607,15 @@ static OPTIMIZE3 void timer_stat_dump(
 		if ((g_opt_flags & OPT_SHOW_WHENCE) && !g_curses_init) {
 			time_t t = (time_t)whence;
 			char *timestr = ctime(&t);
-			char *pos = strchr(timestr, '\n');
+			char *pos;
 
-			if (*pos)
+			if (timestr) {
+				pos = strchr(timestr, '\n');
+			} else {
+				pos = "<unknown>";
+			}
+
+			if (pos)
 				*pos = '\0';
 			es_printf("Timestamp: %s, Total Run Duration: "
 				"%.1f secs\n", timestr, time_delta);
@@ -1581,9 +1642,11 @@ static char *read_events(const double time_end)
 	size_t size;
 
 	if (UNLIKELY(g_get_events_buf == NULL)) {
-		if ((g_get_events_buf = malloc(EVENT_BUF_SIZE << 1)) == NULL)
+		g_get_events_buf = malloc(EVENT_BUF_SIZE << 1);
+		if (UNLIKELY(!g_get_events_buf)) {
 			err_abort("Cannot read %s, out of memory\n",
 				g_sys_tracing_pipe);
+		}
 
 		get_events_size = (EVENT_BUF_SIZE << 1);
 	}
@@ -1633,7 +1696,7 @@ static char *read_events(const double time_end)
 
 			get_events_size += (EVENT_BUF_SIZE << 1);
 			tmpptr = realloc(g_get_events_buf, get_events_size + 1);
-			if (!tmpptr) {
+			if (UNLIKELY(!tmpptr)) {
 				(void)close(fd);
 				err_abort("Cannot read %s, out of memory\n",
 					g_sys_tracing_pipe);
@@ -1668,10 +1731,8 @@ static void get_events(
 
 	while (*tmpptr) {
 		char *ptr, *eol = tmpptr;
-		char task[64];
-		char task_mangled[68];
-		char func[64];
-		char *cmdline;
+		char func[64], task[64];
+		char *cmdline, *comm;
 		int mask;
 		timer_info_t info;
 
@@ -1686,7 +1747,6 @@ static void get_events(
 		}
 		if (strstr(tmpptr, "hrtimer_start")) {
 			(void)memset(&info, 0, sizeof(info));
-			(void)memset(task, 0, sizeof(task));
 			(void)memset(func, 0, sizeof(func));
 
 			/*
@@ -1721,6 +1781,9 @@ static void get_events(
 		/* Processes without a command line are kernel threads */
 		cmdline = get_pid_cmdline(info.pid);
 		info.kernel_thread = pid_a_kernel_thread(task, info.pid);
+		comm = get_pid_comm(info.pid, info.kernel_thread);
+		if (UNLIKELY(!comm))
+			goto free_next;
 
 		/* Swapper is special, like all corner cases */
 		if (UNLIKELY(strncmp(task, "swapper", 6) == 0))
@@ -1730,21 +1793,9 @@ static void get_events(
 		if (!(g_opt_flags & mask))
 			goto free_next;
 
-		if (info.kernel_thread) {
-			char tmp[sizeof(task)];
-
-			(void)strcpy(tmp, task);
-			tmp[13] = '\0';
-			(void)snprintf(task_mangled, sizeof(task_mangled),
-				"[%s]", tmp);
-		} else {
-			(void)strcpy(task_mangled, task);
-		}
-
 		if (strncmp(task, g_app_name, app_name_len)) {
-			info.task = task;
-			info.cmdline = cmdline ? cmdline : task_mangled;
-			info.task_mangled = task_mangled;
+			info.cmdline = cmdline ? cmdline : comm;
+			info.comm = comm;
 			info.func = func;
 			info.time_total = 0.0;
 			info.total_events = 1;
@@ -1753,6 +1804,7 @@ static void get_events(
 		}
 free_next:
 		free(cmdline);
+		free(comm);
 next:
 		tmpptr = eol;
 	}
@@ -1918,7 +1970,8 @@ int main(int argc, char **argv)
 				errno, strerror(errno));
 	}
 
-	if ((timer_stats = calloc(TABLE_SIZE, sizeof(*timer_stats))) == NULL)
+	timer_stats = calloc(TABLE_SIZE, sizeof(*timer_stats));
+	if (UNLIKELY(!timer_stats))
 		err_abort("Cannot allocate timer stats table\n");
 
 	/* Should really catch signals and set back to zero before we die */
